/*
 * blancoDb
 * Copyright (C) 2004-2005 Yasuo Nakanishi
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 */
package blanco.db.expander.query.invoker;

import java.sql.Connection;
import java.sql.PreparedStatement;

import blanco.db.definition.QueryInvoker;
import blanco.db.expander.query.Finalize;
import blanco.db.expander.query.GetQueryMethod;
import blanco.db.expander.query.PrepareStatementMethod;
import blanco.db.expander.query.PrepareStatementMethod2;
import blanco.db.expander.query.QueryConstructor;
import blanco.db.util.BlancoDbObjectStorage;
import blanco.ig.expander.ClassExpander;
import blanco.ig.expander.Type;
import blanco.ig.expander.Value;
import blanco.ig.expander.field.FieldExpander;
import blanco.ig.expander.implementor.Statement;

/**
 * @author Yasuo Nakanishi
 */
public class QueryInvokerClass extends ClassExpander {
    private BlancoDbObjectStorage storage = null;

    private QueryInvoker _invoker = null;

    public QueryInvokerClass(final BlancoDbObjectStorage storage,
            final Type type, final QueryInvoker invoker) {
        super(type);
        _invoker = invoker;
        this.storage = storage;

        getJavaDoc().addLine(type.getName() + "NX<br>");
        getJavaDoc().addLine("QueryInvokerɑNXłB<br>");
        getJavaDoc().addLine("s^SQLbsOĊeANZT񋟂܂B");
        if (invoker.isSingle()) {
            getJavaDoc().addLine("VO: L (҂鏈1)<br>");
        }
    }

    protected void expandClassStruct() {
        addField(new FieldExpander(new Value(Connection.class, "fConnection")));
        addField(new FieldExpander(new Value(PreparedStatement.class,
                "fStatement")));

        if (storage.getSetting().isLogging()) {
            addImport(new Type("org.apache.commons.logging.LogFactory"));
            final FieldExpander field1 = new FieldExpander(new Type(
                    "org.apache.commons.logging.Log"), "fLog");
            field1.getJavaDoc().addLine("MÔ߂̃IuWFNg<br>");
            field1.getJavaDoc().addLine(
                    " static final ɂȂׂBblancoIg̎dloOɂ");
            field1.setDefaultStatement(new Statement("LogFactory.getLog("
                    + getName() + ".class)"));
            addField(field1);
        }

        addMethod(new QueryConstructor(getType()));
        addMethod(new GetQueryMethod(_invoker.getQuery()));

        addMethod(new PrepareStatementMethod(storage));
        // t.iga
        addMethod(new PrepareStatementMethod2(storage, null));

        // ̓p[^ꍇɂ̂bind܂B
        if (_invoker.getInParameterIterator().hasNext()) {
            addMethod(new SetInvokerInputParameterMethod(storage, _invoker));
        }
        addMethod(new ExecuteUpdateMethod(storage, _invoker));
        if (_invoker.isSingle()) {
            addMethod(new ExecuteSingleUpdateMethod(storage, _invoker));
        }
        addMethod(new CloseMethod(storage));

        addMethod(new Finalize(storage, getName()));
    }
}